==> 0.1.2.tar.gz: lib/fst.dart <==
import 'dart:async';

import 'package:flutter/widgets.dart';
import 'package:state_tree/state_tree.dart';

export 'package:state_tree/state_tree.dart';

class _StateTreeLeaf extends StateTreeLeaf implements UpdateReason {
  _StateTreeLeaf(super.parent, super.dependency);

  @override
  _StateTreeNode get parent => super.parent as _StateTreeNode;
}

class _StreamLeaf<T> extends _StateTreeLeaf {
  final Stream Function() streamFactory;
  late StreamSubscription subscription;
  Result<T> value = Pending<T>();

  _StreamLeaf(super.parent, super.dependency, this.streamFactory);

  @override
  void init() {
    super.init();

    final stream = streamFactory();
    subscription = stream.listen((v) {
      value = Success(v);
      parent.signalUpdate(this);
    });
  }

  @override
  _StreamLeaf<T> update(_StreamLeaf<T> leaf) {
    dispose();
    super.update(leaf);

    return leaf..init();
  }

  @override
  void dispose() {
    subscription.cancel();
    super.dispose();
  }
}

class _StateLeaf<T> extends _StateTreeLeaf {
  final T Function() valueFactory;
  late T value;

  _StateLeaf(super.parent, super.dependency, this.valueFactory);

  @override
  void init() {
    super.init();
    value = valueFactory();
  }

  void setState(T value) {
    if (value == this.value) return;
    this.value = value;
    parent.signalUpdate(this);
  }

  @override
  _StateLeaf update(_StateLeaf leaf) {
    leaf.init();
    value = leaf.value;

    super.update(leaf);

    return this;
  }
}

class _ProviderLeaf<T> extends _StateTreeLeaf {
  T value;
  Object? token;

  _ProviderLeaf(super.parent, super.dependency, this.value, this.token);

  @override
  void init() {
    super.init();

    if (parent.element._providerLeafs.containsKey((T, token))) {
      throw StateError(
        'Provider already exists for $T'
        '${token == null ? '.' : 'with token $token.'}',
      );
    }
    parent.element._providerLeafs[(T, token)] = this;
  }

  @override
  _ProviderLeaf<T> update(_ProviderLeaf<T> leaf) {
    dispose();
    leaf.init();

    super.update(leaf);
    return leaf;
  }

  @override
  void dispose() {
    parent.element._providerLeafs.remove((T, token));
    super.dispose();
  }
}

class _ConsumerLeaf<T, K> extends _StateTreeLeaf {
  final Object? token;
  final T Function(K value)? _query;

  late (Type, Object?) providerKey = (K, token);
  late T value;

  _ConsumerLeaf(super.parent, super.dependency, this._query, this.token);

  @override
  void init() {
    super.init();
    value = query(
      _Provider.of<T, K>(parent.element, this, query, token: token).value,
    );
  }

  T _identity(K value) => value as T;

  T query(K value) {
    return (_query ?? _identity)(value);
  }

  bool shouldUpdate(K oldValue, K newValue) {
    final a = query(oldValue);
    final b = query(newValue);

    final shouldUpdate = a != b;

    if (shouldUpdate) {
      value = b;
    }

    return shouldUpdate;
  }
}

class _StateTreeNode implements StateTreeNode {
  final _StateTreeWidgetElement element;

  _StateTreeNode(this.element);

  @override
  void signalUpdate(UpdateReason reason) {
    element.markNeedsBuild();
  }

  @override
  Result<T> stream<T>(
    Stream<T> Function() streamFactory, [
    Record dependency = const (),
  ]) {
    final leaf = _StreamLeaf<T>(this, dependency, streamFactory);
    return element._next(leaf).value;
  }

  @override
  (T, void Function(T)) state<T>(
    T Function() valueFactory, [
    Record dependency = const (),
  ]) {
    final leaf = _StateLeaf(this, dependency, valueFactory);
    final mountedLeaf = element._next(leaf);

    return (mountedLeaf.value, mountedLeaf.setState);
  }

  @override
  void provide<T>(T value, {Record? dependency, Object? token}) {
    final leaf = _ProviderLeaf<T>(this, dependency ?? (value,), value, token);
    element._next(leaf);
  }

  @override
  T consume<T>({Object? token}) {
    final leaf = _ConsumerLeaf<T, T>(this, const (), null, token);
    return element._next(leaf).value;
  }

  @override
  T query<T, K>(T Function(K value) query, {Object? token}) {
    final leaf = _ConsumerLeaf<T, K>(this, const (), query, token);
    return element._next(leaf).value;
  }

  @override
  void effect(void Function() effect, [Record dependency = const ()]) {
    final leaf = _EffectLeaf(this, dependency, effect);
    element._next(leaf);
  }
}

abstract class StateTreeWidget extends Widget {
  const StateTreeWidget({Key? key}) : super(key: key);

  Widget build(BuildContext context, StateTreeNode n);

  @override
  Element createElement() {
    return _StateTreeWidgetElement(this);
  }
}

class _StateTreeWidgetElement extends ComponentElement {
  late final node = _StateTreeNode(this);
  int cursor = 0;
  final _leafs = <StateTreeLeaf>[];
  final _providerLeafs = <(Type, Object?), _ProviderLeaf>{};

  _StateTreeWidgetElement(StateTreeWidget widget) : super(widget);

  @override
  StateTreeWidget get widget => super.widget as StateTreeWidget;

  T _next<T extends StateTreeLeaf>(T leaf) {
    if (cursor < _leafs.length) {
      final existing = _leafs[cursor];
      if (existing.dependency == leaf.dependency) {
        cursor++;
        return existing as T;
      } else {
        final newLeaf = existing.update(leaf);
        _leafs[cursor] = newLeaf;
        cursor++;
        return newLeaf as T;
      }
    }

    leaf.init();
    _leafs.add(leaf);
    cursor++;

    return leaf;
  }

  @override
  Widget build() {
    cursor = 0;

    final child = widget.build(this, node);

    for (int j = cursor; j < _leafs.length; j++) {
      final leaf = _leafs[j];
      leaf.dispose();
    }

    _leafs.removeRange(cursor, _leafs.length);

    for (var element in _leafs) {
      element.cleanup();
    }

    if (_providerLeafs.isNotEmpty) {
      return _Provider(
        leafs: {..._providerLeafs},
        parentElement: this,
        child: child,
      );
    } else {
      return child;
    }
  }

  @override
  void unmount() {
    for (final leaf in _leafs) {
      leaf.dispose();
    }

    super.unmount();
  }
}

class _Provider extends InheritedModel<_ConsumerLeaf> {
  final Map<(Type, Object?), _ProviderLeaf> leafs;
  final Element parentElement;

  const _Provider({
    required super.child,
    required this.leafs,
    required this.parentElement,
  });

  @override
  bool updateShouldNotify(_Provider oldWidget) {
    var shouldNotify =
        oldWidget.leafs.keys.toSet().difference(leafs.keys.toSet()).isNotEmpty;

    if (shouldNotify) return shouldNotify;

    for (final key in leafs.keys) {
      final oldLeaf = oldWidget.leafs[key];
      final newLeaf = leafs[key];

      if (oldLeaf!.value != newLeaf!.value) {
        return true;
      }
    }

    return false;
  }

  @override
  bool updateShouldNotifyDependent(
    _Provider oldWidget,
    Set<_ConsumerLeaf> dependencies,
  ) {
    final shouldNotify = dependencies.any((e) {
      final key = e.providerKey;

      final oldLeaf = oldWidget.leafs[key];
      final newLeaf = leafs[key];

      if (oldLeaf == null || newLeaf == null) return true;

      return e.shouldUpdate(oldLeaf.value, newLeaf.value);
    });

    return shouldNotify;
  }

  static _ProviderLeaf<K> of<T, K>(
    Element element,
    _ConsumerLeaf<T, K> aspect,
    T Function(K value) query, {
    Object? token,
  }) {
    Element? e = element;

    while (true) {
      final ie = e?.getElementForInheritedWidgetOfExactType<_Provider>();
      final model = ie?.widget as _Provider?;

      if (model == null) {
        throw StateError(
          'No provider found for $K'
          '${token == null ? '.' : 'with token $token'}',
        );
      }

      final leaf = model.leafs[(K, token)];

      if (leaf == null) {
        e = model.parentElement;
        continue;
      }

      element.dependOnInheritedElement(ie!, aspect: aspect);

      return leaf as _ProviderLeaf<K>;
    }
  }
}

class _EffectLeaf extends StateTreeLeaf {
  void Function() effect;

  _EffectLeaf(super.parent, super.dependency, this.effect);

  @override
  void init() {
    super.init();
    effect();
  }

  @override
  _EffectLeaf update(_EffectLeaf leaf) {
    effect = leaf.effect;
    effect();
    super.update(leaf);

    return this;
  }
}

class StateTreeBuilder extends StateTreeWidget {
  final Widget Function(
    BuildContext context,
    StateTreeNode n,
    Widget? child,
  ) builder;

  final Widget? child;

  const StateTreeBuilder({
    Key? key,
    required this.builder,
    this.child,
  }) : super(key: key);

  @override
  Widget build(BuildContext context, StateTreeNode n) {
    return builder(context, n, child);
  }
}

==> 0.2.3.tar.gz: lib/fst.dart <==
import 'package:flutter/widgets.dart';
import 'package:state_tree_interface/state_tree_interface.dart';

export 'package:state_tree_interface/state_tree_interface.dart';

class _StateTreeLeaf extends StateTreeLeaf implements UpdateReason {
  _StateTreeLeaf(super.parent, super.dependency);

  @override
  _StateTreeNode get parent => super.parent as _StateTreeNode;
}

class _StateLeaf<T> extends _StateTreeLeaf {
  T value;
  _StateLeaf(StateTreeNode parent, this.value) : super(parent, const Object());

  void setState(T value) {
    if (value == this.value) return;
    this.value = value;
    parent.signalUpdate(this);
  }

  @override
  _StateLeaf update(_StateLeaf newLeaf) {
    value = newLeaf.value;

    super.update(newLeaf);
    return this;
  }
}

class _ProviderLeaf<T> extends _StateTreeLeaf {
  T value;
  Object? token;

  _ProviderLeaf(super.parent, super.dependency, this.value, this.token);

  @override
  void init() {
    super.init();

    if (parent.element._providerLeafs.containsKey((T, token))) {
      throw StateError(
        'Provider already exists for $T'
        '${token == null ? '.' : 'with token $token.'}',
      );
    }

    parent.element._providerLeafs[(T, token)] = this;
  }

  @override
  _ProviderLeaf<T> update(_ProviderLeaf<T> leaf) {
    dispose();
    leaf.init();

    super.update(leaf);
    return leaf;
  }

  @override
  void dispose() {
    parent.element._providerLeafs.remove((T, token));
    super.dispose();
  }
}

class _ConsumerLeaf<T, K> extends _StateTreeLeaf {
  final Object? token;
  final T Function(K value)? _query;

  late (Type, Object?) providerKey = (K, token);
  late T value;

  final bool subscribe;

  _ConsumerLeaf(
    StateTreeNode parent,
    this._query, {
    this.token,
    this.subscribe = true,
  }) : super(parent, const ());

  @override
  void init() {
    super.init();
    value = query(
      _Provider.of<T, K>(
        parent.element,
        aspect: this,
        token: token,
        subscribe: subscribe,
      ).value,
    );
  }

  T _identity(K value) => value as T;

  T query(K value) {
    return (_query ?? _identity)(value);
  }

  bool shouldUpdate(K oldValue, K newValue) {
    final a = query(oldValue);
    final b = query(newValue);

    final shouldUpdate = a != b;

    if (shouldUpdate) {
      value = b;
    }

    return shouldUpdate;
  }
}

class _StateTreeNode implements StateTreeNode {
  final _StateTreeWidgetElement element;

  _StateTreeNode(this.element);

  @override
  void signalUpdate(UpdateReason reason) {
    element.markNeedsBuild();
  }

  @override
  (T, SetState<T>) state<T>(T initialState) {
    final leaf = _StateLeaf(this, initialState);
    final mountedLeaf = element._next(leaf);

    return (mountedLeaf.value, mountedLeaf.setState);
  }

  @override
  void provide<T>(T value, {Object? dependency, Object? token}) {
    final leaf = _ProviderLeaf<T>(this, dependency ?? (value,), value, token);
    element._next(leaf);
  }

  @override
  T consume<T>({Object? token}) {
    final leaf = _ConsumerLeaf<T, T>(this, null, token: token);
    return element._next(leaf).value;
  }

  @override
  T read<T>({Object? token}) {
    return _Provider.of<T, T>(
      element,
      token: token,
    ).value;
  }

  @override
  T query<T, K>(T Function(K value) query, {Object? token}) {
    final leaf = _ConsumerLeaf<T, K>(this, query, token: token);
    return element._next(leaf).value;
  }

  @override
  void effect(void Function()? Function() effect, {Object? dependency}) {
    final leaf = _EffectLeaf(this, dependency ?? const Object(), effect);
    element._next(leaf);
  }

  LateResult<T> stream<T>(Stream<T> stream, {Object? dependency}) {
    final (v, setValue) = state(Late<T>.pending());

    effect(
      () {
        final subscription = stream.listen(
          (v) => setValue(Late<T>.success(v)),
          onError: (err) => setValue(Late<T>.failure(err)),
        );

        return () => subscription.cancel();
      },
      dependency: dependency,
    );

    return v;
  }
}

abstract class _StateTreeWidget extends Widget {
  const _StateTreeWidget({Key? key}) : super(key: key);

  Widget build(BuildContext context, StateTreeNode n);

  @override
  Element createElement() {
    return _StateTreeWidgetElement(this);
  }
}

class _StateTreeWidgetElement extends ComponentElement {
  late final node = _StateTreeNode(this);
  int cursor = 0;
  final _leafs = <StateTreeLeaf>[];
  final _providerLeafs = <(Type, Object?), _ProviderLeaf>{};

  _StateTreeWidgetElement(_StateTreeWidget widget) : super(widget);

  @override
  _StateTreeWidget get widget => super.widget as _StateTreeWidget;

  bool _isBuilding = false;

  T _next<T extends StateTreeLeaf>(T leaf) {
    if (_isBuilding) {
      throw StateError(
        '$leaf could only be used synchronously during build.',
      );
    }

    if (cursor < _leafs.length) {
      final existing = _leafs[cursor];
      if (existing.dependency == leaf.dependency) {
        cursor++;
        return existing as T;
      } else {
        final newLeaf = existing.update(leaf);
        _leafs[cursor] = newLeaf;
        cursor++;
        return newLeaf as T;
      }
    }

    leaf.init();
    _leafs.add(leaf);
    cursor++;

    return leaf;
  }

  @override
  Widget build() {
    cursor = 0;

    _isBuilding = true;
    final child = widget.build(this, node);
    _isBuilding = false;

    for (int j = cursor; j < _leafs.length; j++) {
      final leaf = _leafs[j];
      leaf.dispose();
    }

    _leafs.removeRange(cursor, _leafs.length);

    for (var element in _leafs) {
      element.cleanup();
    }

    if (_providerLeafs.isNotEmpty) {
      return _Provider(
        leafs: {..._providerLeafs},
        parentElement: this,
        child: child,
      );
    } else {
      return child;
    }
  }

  @override
  void unmount() {
    for (final leaf in _leafs) {
      leaf.dispose();
    }

    super.unmount();
  }
}

class _Provider extends InheritedModel<_ConsumerLeaf> {
  final Map<(Type, Object?), _ProviderLeaf> leafs;
  final Element parentElement;

  const _Provider({
    required super.child,
    required this.leafs,
    required this.parentElement,
  });

  @override
  bool updateShouldNotify(_Provider oldWidget) {
    final oldKeys = oldWidget.leafs.keys.toSet();
    final newKeys = leafs.keys.toSet();

    final shouldNotify = oldKeys.difference(newKeys).isNotEmpty;

    if (shouldNotify) return shouldNotify;

    for (final key in leafs.keys) {
      final oldLeaf = oldWidget.leafs[key];
      final newLeaf = leafs[key];

      if (oldLeaf!.value != newLeaf!.value) {
        return true;
      }
    }

    return false;
  }

  @override
  bool updateShouldNotifyDependent(
    _Provider oldWidget,
    Set<_ConsumerLeaf> dependencies,
  ) {
    final shouldNotify = dependencies.any((e) {
      final key = e.providerKey;

      final oldLeaf = oldWidget.leafs[key];
      final newLeaf = leafs[key];

      if (oldLeaf == null || newLeaf == null) return true;

      return e.shouldUpdate(oldLeaf.value, newLeaf.value);
    });

    return shouldNotify;
  }

  static _ProviderLeaf<K> of<T, K>(
    Element element, {
    _ConsumerLeaf<T, K>? aspect,
    bool subscribe = false,
    Object? token,
  }) {
    Element? e = element;

    while (true) {
      final ie = e?.getElementForInheritedWidgetOfExactType<_Provider>();
      final model = ie?.widget as _Provider?;

      if (model == null) {
        throw StateError(
          'No provider found for $K'
          '${token == null ? '.' : 'with token $token'}',
        );
      }

      final leaf = model.leafs[(K, token)];

      if (leaf == null) {
        e = model.parentElement;
        continue;
      }

      if (subscribe) element.dependOnInheritedElement(ie!, aspect: aspect);

      return leaf as _ProviderLeaf<K>;
    }
  }
}

class _EffectLeaf extends _StateTreeLeaf {
  final void Function()? Function() effect;
  void Function()? unsubscribe;

  _EffectLeaf(super.parent, super.dependency, this.effect);

  @override
  void init() {
    super.init();
    unsubscribe = effect();
  }

  @override
  _EffectLeaf update(_EffectLeaf leaf) {
    dispose();
    leaf.init();

    super.update(leaf);
    return leaf;
  }

  @override
  void dispose() {
    unsubscribe?.call();
    super.dispose();
  }
}

class StateTreeBuilder extends _StateTreeWidget {
  final Widget Function(
    BuildContext context,
    StateTreeNode n,
    Widget child,
  ) builder;

  final Widget child;

  const StateTreeBuilder(
    this.builder, {
    Key? key,
    this.child = const SizedBox(),
  }) : super(key: key);

  @override
  Widget build(BuildContext context, StateTreeNode n) {
    return builder(context, n, child);
  }
}
